# This script tests recipient validation using the flowchart in the
# documentation folder.  It follows this path through the flowchart:
#   RELAYCLIENT is not set (f01__is_relayclient_set)
#   recipient has domain name (f02__does_recipient_have_domain_name)
#   domain is in rcpthosts (f05__is_domain_in_rcpthosts)
#   recipient does not contain a(nother) percent sign (f07__does_recipient_contain_percent)
#   domain is in locals (f09__is_domain_in_locals)
#   cdb contains recipient exact match (f12__does_cdb_contain_exact_match)
#   home directory exists (f14__does_exact_match_home_directory_exist)
#   home directory is readable (f15__is_exact_match_home_directory_readable)
#   home directory is not world-writable (f16__is_exact_match_home_directory_world_writable)
#   home directory is not sticky (f17__is_exact_match_home_directory_sticky)
#   constructed filename does not exist (f18__does_exact_match_filename_exist)
#   constructed filename does not exist (f19__does_exact_match_filename_exist_default)
#   dash field is not empty (f20__is_exact_match_dash_empty)
# The message should be rejected

export TCPREMOTEIP=11.22.33.44
unset GREP_TARGET
unset REJECT_GREP_TARGET
unset TEMP_FILENAME
unset HOMEDIR
unset DASH
unset EXT
unset REAL_RELAYCLIENT
unset REAL_USERNAME
unset REAL_DOMAIN
unset REAL_ADDRESS
unset TO_USERNAME
unset TO_DOMAIN
unset TO_ADDRESS
unset TEMP_USERNAME
unset WILDCARD_DOT
unset WILDCARD_DASH
unset WILDCARD_DASH2
unset STRLEN_REAL_USERNAME
unset PATTERN
unset WILDCARD_ADDITION
unset IN_MORERCPTHOSTS
unset IN_RCPTHOSTS
unset PERCENTHACK_USERNAME_ADDITION
unset VIRTUALDOMAINS_USERNAME_ADDITION
FROM_ADDRESS=test-${TEST_NUM}.clcmc.ocpcq@example.com
REAL_USERNAME=s-crc
REAL_DOMAIN=sc.test
REAL_ADDRESS=${REAL_USERNAME}@${REAL_DOMAIN}
TO_USERNAME=${REAL_USERNAME}
TO_DOMAIN=${REAL_DOMAIN}
TO_ADDRESS=${REAL_ADDRESS}
TEMP_USERNAME=${REAL_USERNAME}
VALID_DOMAIN=valid-domain-tcucv-cwcxc-yczc0.test
VALID_USERNAME=valid-user-c1c2c-3c4c5-c6c7c
VALID_ADDRESS=${VALID_USERNAME}@${VALID_DOMAIN}
INVALID_USERNAME=invalid-user-8c9c9-c8c7c-6c5c4
INVALID_ADDRESS=${INVALID_USERNAME}@${VALID_DOMAIN}
VALID_HOMEDIR=${TMPDIR}/${TEST_NUM}-valid_home
VALID_GREP_TARGET=${VALID_HOMEDIR}/mbox
mkdir -p ${VALID_HOMEDIR}
chmod 700 ${VALID_HOMEDIR}
chown -R $6 ${VALID_HOMEDIR}
chgrp -R $7 ${VALID_HOMEDIR}
echo ${VALID_HOMEDIR}/mbox > ${VALID_HOMEDIR}/.qmail
touch ${VALID_HOMEDIR}/mbox
chmod 600 ${VALID_HOMEDIR}/mbox
chown $6 ${VALID_HOMEDIR}/mbox
chgrp $7 ${VALID_HOMEDIR}/mbox


################################################################################
# RELAYCLIENT is not set (f01__is_relayclient_set)
LEVEL_f01__is_relayclient_set=1
export -n RELAYCLIENT

################################################################################
# domain is in rcpthosts (f05__is_domain_in_rcpthosts)
LEVEL_f05__is_domain_in_rcpthosts=1
IN_RCPTHOSTS="true"
cp /var/qmail/control/rcpthosts ${TMPDIR}/${TEST_NUM}-f05_rcpthosts.bu.${LEVEL_f05__is_domain_in_rcpthosts}
echo ${REAL_DOMAIN} > ${TMPDIR}/${TEST_NUM}-f05_rcpthosts.${LEVEL_f05__is_domain_in_rcpthosts}
cp ${TMPDIR}/${TEST_NUM}-f05_rcpthosts.${LEVEL_f05__is_domain_in_rcpthosts} /var/qmail/control/rcpthosts

################################################################################
# domain is in locals (f09__is_domain_in_locals)
LEVEL_f09__is_domain_in_locals=1
cp /var/qmail/control/locals ${TMPDIR}/${TEST_NUM}-f09_locals.bu.${LEVEL_f09__is_domain_in_locals}
echo ${REAL_DOMAIN} > ${TMPDIR}/${TEST_NUM}-f09_locals.${LEVEL_f09__is_domain_in_locals}
echo ${TO_DOMAIN}${REAL_RELAYCLIENT} >> ${TMPDIR}/${TEST_NUM}-f09_locals.${LEVEL_f09__is_domain_in_locals}
echo ${VALID_DOMAIN} >> ${TMPDIR}/${TEST_NUM}-f09_locals.${LEVEL_f09__is_domain_in_locals}
cp ${TMPDIR}/${TEST_NUM}-f09_locals.${LEVEL_f09__is_domain_in_locals} /var/qmail/control/locals

################################################################################
# cdb contains recipient exact match (f12__does_cdb_contain_exact_match)
LEVEL_f12__does_cdb_contain_exact_match=1
cp /var/qmail/users/assign ${TMPDIR}/${TEST_NUM}-f12_assign.bu.${LEVEL_f12__does_cdb_contain_exact_match}
cat ${TMPDIR}/${TEST_NUM}-f12_assign.bu.${LEVEL_f12__does_cdb_contain_exact_match} | sed -e "/^=${REAL_USERNAME}:.*$/d" -e "/^\.$/d" > ${TMPDIR}/${TEST_NUM}-f12_assign.${LEVEL_f12__does_cdb_contain_exact_match}
PATTERN=${REAL_USERNAME}
USER=${REAL_USERNAME}
HOMEDIR=${TMPDIR}/${TEST_NUM}-home.${LEVEL_f12__does_cdb_contain_exact_match}.2m2o2
DASH=-DASH${LEVEL_f12__does_cdb_contain_exact_match}.p2q2r-
DASH_EMPTY=""
FILE_EXT=EXT${LEVEL_f12__does_cdb_contain_exact_match}.2s2t2
EXT=`echo ${FILE_EXT} | tr '[:upper:]' '[:lower:]' | sed -e "s/\./:/g"`
TEMP_FILENAME=${HOMEDIR}/.qmail${DASH}${EXT}
echo "=${PATTERN}:${REAL_USERNAME}:$6:$7:${HOMEDIR}:${DASH}:${FILE_EXT}:" >> ${TMPDIR}/${TEST_NUM}-f12_assign.${LEVEL_f12__does_cdb_contain_exact_match}
echo "=${VALID_USERNAME}:${VALID_USERNAME}:$6:$7:${VALID_HOMEDIR}:::" >> ${TMPDIR}/${TEST_NUM}-f12_assign.${LEVEL_f12__does_cdb_contain_exact_match}
echo "." >> ${TMPDIR}/${TEST_NUM}-f12_assign.${LEVEL_f12__does_cdb_contain_exact_match}
cp ${TMPDIR}/${TEST_NUM}-f12_assign.${LEVEL_f12__does_cdb_contain_exact_match} /var/qmail/users/assign
/var/qmail/bin/qmail-newu

################################################################################
# home directory exists (f14__does_exact_match_home_directory_exist)
LEVEL_f14__does_exact_match_home_directory_exist=1
mkdir -p ${HOMEDIR}
chmod 700 ${HOMEDIR}
chown -R $6 ${HOMEDIR}
chgrp -R $7 ${HOMEDIR}

################################################################################
# home directory is readable (f15__is_exact_match_home_directory_readable)
LEVEL_f15__is_exact_match_home_directory_readable=1
chmod u+r ${HOMEDIR}

################################################################################
# home directory is not world-writable (f16__is_exact_match_home_directory_world_writable)
LEVEL_f16__is_exact_match_home_directory_world_writable=1
eval `${STMODE_PATH} ${HOMEDIR}`
EXACT_HOMEDIR_WRITABLE_MODE[${LEVEL_f16__is_exact_match_home_directory_world_writable}]=${st_mode}
chmod o-w ${HOMEDIR}

################################################################################
# home directory is not sticky (f17__is_exact_match_home_directory_sticky)
LEVEL_f17__is_exact_match_home_directory_sticky=1
eval `${STMODE_PATH} ${HOMEDIR}`
EXACT_HOMEDIR_STICKY_MODE[${LEVEL_f17__is_exact_match_home_directory_sticky}]=${st_mode}
chmod -t ${HOMEDIR}

################################################################################
# constructed filename does not exist (f18__does_exact_match_filename_exist)
LEVEL_f18__does_exact_match_filename_exist=1
TEMP_FILENAME=${HOMEDIR}/.qmail${DASH}default

################################################################################
# constructed filename does not exist (f19__does_exact_match_filename_exist_default)
LEVEL_f19__does_exact_match_filename_exist_default=1
rm -f ${TEMP_FILENAME}

################################################################################
# final steps
if [ -z "${GREP_TARGET}" ]
then
  GREP_TARGET=${REJECT_GREP_TARGET}
fi

################################################################################
# empty queue
echo Emptying queue and restarting qmail...
svc -d /service/qmail-send
svc -d /service/qmail-smtpd
for i in 2 1 0
do
  output=`ps ax | grep qmail-s[em][nt][dp] | sed -e "/^.*grep.*$/d" -e "/^.*tail.*$/d" -e "/^.*supervise.*$/d" -e "/^.*tcpserver.*$/d"`
  if [ -z "${output}" ]
  then
    break
  else
    ${SLEEP_PATH} 1
  fi
done
if [ ! -z "${output}" ]
then
  svc -k /service/qmail-send
  svc -k /service/qmail-smtpd
fi
rm -f `find /var/qmail/queue/mess /var/qmail/queue/info /var/qmail/queue/local /var/qmail/queue/remote /var/qmail/queue/bounce /var/qmail/queue/intd /var/qmail/queue/todo -type f -print`
svc -u /service/qmail-send
svc -u /service/qmail-smtpd


################################################################################
# create input file
echo "HELO me" > ${TMPDIR}/${TEST_NUM}-input.txt
echo "MAIL FROM:<${FROM_ADDRESS}>" >> ${TMPDIR}/${TEST_NUM}-input.txt
echo "RCPT TO:<${TO_ADDRESS}>" >> ${TMPDIR}/${TEST_NUM}-input.txt
echo "QUIT" >> ${TMPDIR}/${TEST_NUM}-input.txt

mkdir -p ${TMPDIR}/${TEST_NUM}-config.d/_sender_/com

echo "reject-recipient=invalid" >> ${TMPDIR}/${TEST_NUM}-config.d/_sender_/com/example
echo "reject-recipient=unavailable" >> ${TMPDIR}/${TEST_NUM}-config.d/_sender_/com/example
echo "rejection-text-recipient-invalid=Foo Bar Baz Qux" >> ${TMPDIR}/${TEST_NUM}-config.d/_sender_/com/example

################################################################################
# run test
echo "${SENDRECV_PATH} -t 30 -r 221 -w 0 -- ${SPAMDYKE_PATH} --log-target stderr -linfo --recipient-validation-command ${SPAMDYKE_QRV_PATH} --config-dir ${TMPDIR}/${TEST_NUM}-config.d ${QMAIL_CMDLINE} < ${TMPDIR}/${TEST_NUM}-input.txt > ${TMPDIR}/${TEST_NUM}-output.txt 2>&1"
${SENDRECV_PATH} -t 30 -r 221 -w 0 -- ${SPAMDYKE_PATH} --log-target stderr -linfo --recipient-validation-command ${SPAMDYKE_QRV_PATH} --config-dir ${TMPDIR}/${TEST_NUM}-config.d ${QMAIL_CMDLINE} < ${TMPDIR}/${TEST_NUM}-input.txt > ${TMPDIR}/${TEST_NUM}-output.txt 2>&1

output=`grep "554 Refused. The recipient address does not exist." ${TMPDIR}/${TEST_NUM}-output.txt`
if [ -z "${output}" ]
then
  output=`grep "554 Foo Bar Baz Qux" ${TMPDIR}/${TEST_NUM}-output.txt`
  if [ ! -z "${output}" ]
  then
    output=`grep "DENIED_INVALID_RECIPIENT from: ${FROM_ADDRESS} to: ${TO_ADDRESS}" ${TMPDIR}/${TEST_NUM}-output.txt`
    if [ ! -z "${output}" ]
    then
      outcome="success"
    else
      echo OUTPUT IN tmp/${TEST_NUM}-output.txt:
      cat ${TMPDIR}/${TEST_NUM}-output.txt

      outcome="failure"
    fi
  else
    echo OUTPUT IN tmp/${TEST_NUM}-output.txt:
    cat ${TMPDIR}/${TEST_NUM}-output.txt

    outcome="failure"
  fi
else
  echo OUTPUT IN tmp/${TEST_NUM}-output.txt:
  cat ${TMPDIR}/${TEST_NUM}-output.txt

  outcome="failure"
fi

################################################################################
# home directory is not sticky (f17__is_exact_match_home_directory_sticky)
chmod ${EXACT_HOMEDIR_STICKY_MODE[${LEVEL_f17__is_exact_match_home_directory_sticky}]} ${HOMEDIR}
LEVEL_f17__is_exact_match_home_directory_sticky=0

################################################################################
# home directory is not world-writable (f16__is_exact_match_home_directory_world_writable)
chmod ${EXACT_HOMEDIR_WRITABLE_MODE[${LEVEL_f16__is_exact_match_home_directory_world_writable}]} ${HOMEDIR}
LEVEL_f16__is_exact_match_home_directory_world_writable=0

################################################################################
# cdb contains recipient exact match (f12__does_cdb_contain_exact_match)
cp ${TMPDIR}/${TEST_NUM}-f12_assign.bu.${LEVEL_f12__does_cdb_contain_exact_match} /var/qmail/users/assign
/var/qmail/bin/qmail-newu
LEVEL_f12__does_cdb_contain_exact_match=0

################################################################################
# domain is in locals (f09__is_domain_in_locals)
cp ${TMPDIR}/${TEST_NUM}-f09_locals.bu.${LEVEL_f09__is_domain_in_locals} /var/qmail/control/locals
LEVEL_f09__is_domain_in_locals=0

################################################################################
# domain is in rcpthosts (f05__is_domain_in_rcpthosts)
cp ${TMPDIR}/${TEST_NUM}-f05_rcpthosts.bu.${LEVEL_f05__is_domain_in_rcpthosts} /var/qmail/control/rcpthosts
LEVEL_f05__is_domain_in_rcpthosts=0


echo Restarting qmail...
svc -d /service/qmail-send
svc -d /service/qmail-smtpd
for i in 2 1 0
do
  output=`ps ax | grep qmail-s[em][nt][dp] | sed -e "/^.*grep.*$/d" -e "/^.*tail.*$/d" -e "/^.*supervise.*$/d" -e "/^.*tcpserver.*$/d"`
  if [ -z "${output}" ]
  then
    break
  else
    ${SLEEP_PATH} 1
  fi
done
if [ ! -z "${output}" ]
then
  svc -k /service/qmail-send
  svc -k /service/qmail-smtpd
fi
svc -u /service/qmail-send
svc -u /service/qmail-smtpd

