# This script tests recipient validation using the flowchart in the
# documentation folder.  It follows this path through the flowchart:
#   RELAYCLIENT is not set (f01__is_relayclient_set)
#   recipient has domain name (f02__does_recipient_have_domain_name)
#   domain is in rcpthosts (f05__is_domain_in_rcpthosts)
#   recipient does not contain a(nother) percent sign (f07__does_recipient_contain_percent)
#   domain is in locals (f09__is_domain_in_locals)
#   cdb contains recipient exact match (f12__does_cdb_contain_exact_match)
#   home directory exists (f14__does_exact_match_home_directory_exist)
#   home directory is readable (f15__is_exact_match_home_directory_readable)
#   home directory is not world-writable (f16__is_exact_match_home_directory_world_writable)
#   home directory is not sticky (f17__is_exact_match_home_directory_sticky)
#   constructed filename does not exist (f18__does_exact_match_filename_exist)
#   constructed filename exists (f19__does_exact_match_filename_exist_default)
#   file is not world-writeable (f33__is_file_world_writable)
#   file is not empty (f34__is_file_empty)
#   file is not owner-executable (f35__is_file_owner_executable)
#   first line is not blank (f36__is_first_line_blank)
#   more lines in file (f37__more_lines_in_file)
#   line in file is not a comment (f38__is_line_comment)
#   line in file is not an mbox (f39__is_line_mbox)
#   line in file is not a maildir (f41__is_line_maildir)
#   line in file is not a program, it must be a forward address (f43__is_line_program)
#   forward address is valid (f45__is_forward_address_valid)
# The forward address has no domain name, envnoathost and me do not exist but qmail-me-file tells spamdyke to use a different file
# The message should be accepted

export TCPREMOTEIP=11.22.33.44
unset GREP_TARGET
unset REJECT_GREP_TARGET
unset TEMP_FILENAME
unset HOMEDIR
unset DASH
unset EXT
unset REAL_RELAYCLIENT
unset REAL_USERNAME
unset REAL_DOMAIN
unset REAL_ADDRESS
unset TO_USERNAME
unset TO_DOMAIN
unset TO_ADDRESS
unset TEMP_USERNAME
unset WILDCARD_DOT
unset WILDCARD_DASH
unset WILDCARD_DASH2
unset STRLEN_REAL_USERNAME
unset PATTERN
unset WILDCARD_ADDITION
unset IN_MORERCPTHOSTS
unset IN_RCPTHOSTS
unset PERCENTHACK_USERNAME_ADDITION
unset VIRTUALDOMAINS_USERNAME_ADDITION
FROM_ADDRESS=test-${TEST_NUM}.araqa.paoam@example.com
REAL_USERNAME=s-ala
REAL_DOMAIN=ka.test
REAL_ADDRESS=${REAL_USERNAME}@${REAL_DOMAIN}
TO_USERNAME=${REAL_USERNAME}
TO_DOMAIN=${REAL_DOMAIN}
TO_ADDRESS=${REAL_ADDRESS}
TEMP_USERNAME=${REAL_USERNAME}
VALID_DOMAIN=domain-jaiah-agafa-dacab.test
VALID_USERNAME=user-aaaaa-bacad-afaga
VALID_ADDRESS=${VALID_USERNAME}@${VALID_DOMAIN}
VALID_HOMEDIR=${TMPDIR}/${TEST_NUM}-valid_home
VALID_GREP_TARGET=${VALID_HOMEDIR}/mbox
mkdir -p ${VALID_HOMEDIR}
chmod 700 ${VALID_HOMEDIR}
chown -R $6 ${VALID_HOMEDIR}
chgrp -R $7 ${VALID_HOMEDIR}
echo ${VALID_HOMEDIR}/mbox > ${VALID_HOMEDIR}/.qmail
touch ${VALID_HOMEDIR}/mbox
chmod 600 ${VALID_HOMEDIR}/mbox
chown $6 ${VALID_HOMEDIR}/mbox
chgrp $7 ${VALID_HOMEDIR}/mbox


################################################################################
# RELAYCLIENT is not set (f01__is_relayclient_set)
LEVEL_f01__is_relayclient_set=1
export -n RELAYCLIENT

################################################################################
# domain is in rcpthosts (f05__is_domain_in_rcpthosts)
LEVEL_f05__is_domain_in_rcpthosts=1
IN_RCPTHOSTS="true"
cp /var/qmail/control/rcpthosts ${TMPDIR}/${TEST_NUM}-f05_rcpthosts.bu.${LEVEL_f05__is_domain_in_rcpthosts}
echo ${REAL_DOMAIN} > ${TMPDIR}/${TEST_NUM}-f05_rcpthosts.${LEVEL_f05__is_domain_in_rcpthosts}
cp ${TMPDIR}/${TEST_NUM}-f05_rcpthosts.${LEVEL_f05__is_domain_in_rcpthosts} /var/qmail/control/rcpthosts

################################################################################
# domain is in locals (f09__is_domain_in_locals)
LEVEL_f09__is_domain_in_locals=1
cp /var/qmail/control/locals ${TMPDIR}/${TEST_NUM}-f09_locals.bu.${LEVEL_f09__is_domain_in_locals}
echo ${REAL_DOMAIN} > ${TMPDIR}/${TEST_NUM}-f09_locals.${LEVEL_f09__is_domain_in_locals}
echo ${TO_DOMAIN}${REAL_RELAYCLIENT} >> ${TMPDIR}/${TEST_NUM}-f09_locals.${LEVEL_f09__is_domain_in_locals}
echo ${VALID_DOMAIN} >> ${TMPDIR}/${TEST_NUM}-f09_locals.${LEVEL_f09__is_domain_in_locals}
cp ${TMPDIR}/${TEST_NUM}-f09_locals.${LEVEL_f09__is_domain_in_locals} /var/qmail/control/locals

################################################################################
# cdb contains recipient exact match (f12__does_cdb_contain_exact_match)
LEVEL_f12__does_cdb_contain_exact_match=1
cp /var/qmail/users/assign ${TMPDIR}/${TEST_NUM}-f12_assign.bu.${LEVEL_f12__does_cdb_contain_exact_match}
cat ${TMPDIR}/${TEST_NUM}-f12_assign.bu.${LEVEL_f12__does_cdb_contain_exact_match} | sed -e "/^=${REAL_USERNAME}:.*$/d" -e "/^\.$/d" > ${TMPDIR}/${TEST_NUM}-f12_assign.${LEVEL_f12__does_cdb_contain_exact_match}
PATTERN=${REAL_USERNAME}
USER=${REAL_USERNAME}
HOMEDIR=${TMPDIR}/${TEST_NUM}-home.${LEVEL_f12__does_cdb_contain_exact_match}.dzd0d
DASH=-DASH${LEVEL_f12__does_cdb_contain_exact_match}.1d2d3-
DASH_EMPTY=""
FILE_EXT=EXT${LEVEL_f12__does_cdb_contain_exact_match}.d4d5d
EXT=`echo ${FILE_EXT} | tr '[:upper:]' '[:lower:]' | sed -e "s/\./:/g"`
TEMP_FILENAME=${HOMEDIR}/.qmail${DASH}${EXT}
echo "=${PATTERN}:${REAL_USERNAME}:$6:$7:${HOMEDIR}:${DASH}:${FILE_EXT}:" >> ${TMPDIR}/${TEST_NUM}-f12_assign.${LEVEL_f12__does_cdb_contain_exact_match}
echo "=${VALID_USERNAME}:${VALID_USERNAME}:$6:$7:${VALID_HOMEDIR}:::" >> ${TMPDIR}/${TEST_NUM}-f12_assign.${LEVEL_f12__does_cdb_contain_exact_match}
echo "." >> ${TMPDIR}/${TEST_NUM}-f12_assign.${LEVEL_f12__does_cdb_contain_exact_match}
cp ${TMPDIR}/${TEST_NUM}-f12_assign.${LEVEL_f12__does_cdb_contain_exact_match} /var/qmail/users/assign
/var/qmail/bin/qmail-newu

################################################################################
# home directory exists (f14__does_exact_match_home_directory_exist)
LEVEL_f14__does_exact_match_home_directory_exist=1
mkdir -p ${HOMEDIR}
chmod 700 ${HOMEDIR}
chown -R $6 ${HOMEDIR}
chgrp -R $7 ${HOMEDIR}

################################################################################
# home directory is readable (f15__is_exact_match_home_directory_readable)
LEVEL_f15__is_exact_match_home_directory_readable=1
chmod u+r ${HOMEDIR}

################################################################################
# home directory is not world-writable (f16__is_exact_match_home_directory_world_writable)
LEVEL_f16__is_exact_match_home_directory_world_writable=1
eval `${STMODE_PATH} ${HOMEDIR}`
EXACT_HOMEDIR_WRITABLE_MODE[${LEVEL_f16__is_exact_match_home_directory_world_writable}]=${st_mode}
chmod o-w ${HOMEDIR}

################################################################################
# home directory is not sticky (f17__is_exact_match_home_directory_sticky)
LEVEL_f17__is_exact_match_home_directory_sticky=1
eval `${STMODE_PATH} ${HOMEDIR}`
EXACT_HOMEDIR_STICKY_MODE[${LEVEL_f17__is_exact_match_home_directory_sticky}]=${st_mode}
chmod -t ${HOMEDIR}

################################################################################
# constructed filename does not exist (f18__does_exact_match_filename_exist)
LEVEL_f18__does_exact_match_filename_exist=1
TEMP_FILENAME=${HOMEDIR}/.qmail${DASH}default

################################################################################
# constructed filename exists (f19__does_exact_match_filename_exist_default)
LEVEL_f19__does_exact_match_filename_exist_default=1
touch ${TEMP_FILENAME}
chown $6 ${TEMP_FILENAME}
chgrp $7 ${TEMP_FILENAME}

################################################################################
# file is not world-writeable (f33__is_file_world_writable)
LEVEL_f33__is_file_world_writable=1
eval `${STMODE_PATH} ${TEMP_FILENAME}`
FILE_WRITABLE_MODE[${LEVEL_f33__is_file_world_writable}]=${st_mode}
chmod o-w ${TEMP_FILENAME}

################################################################################
# file is not owner-executable (f35__is_file_owner_executable)
LEVEL_f35__is_file_owner_executable=1
eval `${STMODE_PATH} ${TEMP_FILENAME}`
FILE_EXECUTABLE_MODE[${LEVEL_f35__is_file_owner_executable}]=${st_mode}
chmod u-x ${TEMP_FILENAME}

VALID_USERNAME=`echo ${VALID_ADDRESS} | sed -e "s/@.*//"`
VALID_DOMAIN=`echo ${VALID_ADDRESS} | sed -e "s/[^@]*@//"`

if [ -f /var/qmail/control/envnoathost ]
then
  cp /var/qmail/control/envnoathost ${TMPDIR}/${TEST_NUM}-envnoathost.bu
  rm /var/qmail/control/envnoathost
fi
if [ -f /var/qmail/control/me ]
then
  cp /var/qmail/control/me ${TMPDIR}/${TEST_NUM}-me.bu
  rm /var/qmail/control/me
fi
echo ${VALID_DOMAIN} > ${TMPDIR}/${TEST_NUM}-me

################################################################################
# forward address is valid (f45__is_forward_address_valid)
LEVEL_f45__is_forward_address_valid=1
GREP_TARGET=${VALID_GREP_TARGET}
echo "&${VALID_USERNAME}" >> ${TEMP_FILENAME}

################################################################################
# final steps
if [ -z "${GREP_TARGET}" ]
then
  GREP_TARGET=${REJECT_GREP_TARGET}
fi

################################################################################
# empty queue
echo Emptying queue and restarting qmail...
svc -d /service/qmail-send
svc -d /service/qmail-smtpd
for i in 2 1 0
do
  output=`ps ax | grep qmail-s[em][nt][dp] | sed -e "/^.*grep.*$/d" -e "/^.*tail.*$/d" -e "/^.*supervise.*$/d" -e "/^.*tcpserver.*$/d"`
  if [ -z "${output}" ]
  then
    break
  else
    ${SLEEP_PATH} 1
  fi
done
if [ ! -z "${output}" ]
then
  svc -k /service/qmail-send
  svc -k /service/qmail-smtpd
fi
rm -f `find /var/qmail/queue/mess /var/qmail/queue/info /var/qmail/queue/local /var/qmail/queue/remote /var/qmail/queue/bounce /var/qmail/queue/intd /var/qmail/queue/todo -type f -print`
svc -u /service/qmail-send
svc -u /service/qmail-smtpd


################################################################################
# create input file
echo "HELO me" > ${TMPDIR}/${TEST_NUM}-input.txt
echo "MAIL FROM:<${FROM_ADDRESS}>" >> ${TMPDIR}/${TEST_NUM}-input.txt
echo "RCPT TO:<${TO_ADDRESS}>" >> ${TMPDIR}/${TEST_NUM}-input.txt
echo "DATA" >> ${TMPDIR}/${TEST_NUM}-input.txt
echo "To: <${REAL_ADDRESS}>\r\nFrom: <${FROM_ADDRESS}>\r\nSubject: ${TEST_NUM}\r\n\r\nFOO!\r\n.\r\n" >> ${TMPDIR}/${TEST_NUM}-input.txt
echo "QUIT" >> ${TMPDIR}/${TEST_NUM}-input.txt

echo "recipient-validation-command=${SPAMDYKE_QRV_PATH} --qmail-me-file=${TMPDIR}/${TEST_NUM}-me" >> ${TMPDIR}/${TEST_NUM}-config.txt
echo "reject-recipient=invalid" >> ${TMPDIR}/${TEST_NUM}-config.txt
echo "reject-recipient=unavailable" >> ${TMPDIR}/${TEST_NUM}-config.txt

################################################################################
# run test
echo "${SENDRECV_PATH} -t 30 -r 221 -w 0 -- ${SPAMDYKE_PATH} --log-target stderr -linfo -f ${TMPDIR}/${TEST_NUM}-config.txt ${SMTPDUMMY_PATH} < ${TMPDIR}/${TEST_NUM}-input.txt > ${TMPDIR}/${TEST_NUM}-output.txt 2>&1"
${SENDRECV_PATH} -t 30 -r 221 -w 0 -- ${SPAMDYKE_PATH} --log-target stderr -linfo -f ${TMPDIR}/${TEST_NUM}-config.txt ${SMTPDUMMY_PATH} < ${TMPDIR}/${TEST_NUM}-input.txt > ${TMPDIR}/${TEST_NUM}-output.txt 2>&1

output=`grep "ALLOWED from: ${FROM_ADDRESS} to: ${TO_ADDRESS}" ${TMPDIR}/${TEST_NUM}-output.txt`
if [ ! -z "${output}" ]
then
  unset outcome
  output=`grep "250 DATA END received" ${TMPDIR}/${TEST_NUM}-output.txt`
  if [ ! -z "${output}" ]
  then
    outcome="success"
  else
    echo OUTPUT IN tmp/${TEST_NUM}-output.txt:
    cat ${TMPDIR}/${TEST_NUM}-output.txt

    outcome="failure"
  fi
else
  echo OUTPUT IN tmp/${TEST_NUM}-output.txt:
  cat ${TMPDIR}/${TEST_NUM}-output.txt

  outcome="failure"
fi

rm -f /var/qmail/control/envnoathost
rm -f /var/qmail/control/me
if [ -f ${TMPDIR}/${TEST_NUM}-envnoathost.bu ]
then
  cp ${TMPDIR}/${TEST_NUM}-envnoathost.bu /var/qmail/control/envnoathost
fi
if [ -f ${TMPDIR}/${TEST_NUM}-me.bu ]
then
  cp ${TMPDIR}/${TEST_NUM}-me.bu /var/qmail/control/me
fi

################################################################################
# file is not owner-executable (f35__is_file_owner_executable)
chmod ${FILE_EXECUTABLE_MODE[${LEVEL_f35__is_file_owner_executable}]} ${TEMP_FILENAME}
LEVEL_f35__is_file_owner_executable=0

################################################################################
# file is not world-writeable (f33__is_file_world_writable)
chmod ${FILE_WRITABLE_MODE[${LEVEL_f33__is_file_world_writable}]} ${TEMP_FILENAME}
LEVEL_f33__is_file_world_writable=0

################################################################################
# home directory is not sticky (f17__is_exact_match_home_directory_sticky)
chmod ${EXACT_HOMEDIR_STICKY_MODE[${LEVEL_f17__is_exact_match_home_directory_sticky}]} ${HOMEDIR}
LEVEL_f17__is_exact_match_home_directory_sticky=0

################################################################################
# home directory is not world-writable (f16__is_exact_match_home_directory_world_writable)
chmod ${EXACT_HOMEDIR_WRITABLE_MODE[${LEVEL_f16__is_exact_match_home_directory_world_writable}]} ${HOMEDIR}
LEVEL_f16__is_exact_match_home_directory_world_writable=0

################################################################################
# cdb contains recipient exact match (f12__does_cdb_contain_exact_match)
cp ${TMPDIR}/${TEST_NUM}-f12_assign.bu.${LEVEL_f12__does_cdb_contain_exact_match} /var/qmail/users/assign
/var/qmail/bin/qmail-newu
LEVEL_f12__does_cdb_contain_exact_match=0

################################################################################
# domain is in locals (f09__is_domain_in_locals)
cp ${TMPDIR}/${TEST_NUM}-f09_locals.bu.${LEVEL_f09__is_domain_in_locals} /var/qmail/control/locals
LEVEL_f09__is_domain_in_locals=0

################################################################################
# domain is in rcpthosts (f05__is_domain_in_rcpthosts)
cp ${TMPDIR}/${TEST_NUM}-f05_rcpthosts.bu.${LEVEL_f05__is_domain_in_rcpthosts} /var/qmail/control/rcpthosts
LEVEL_f05__is_domain_in_rcpthosts=0


echo Restarting qmail...
svc -d /service/qmail-send
svc -d /service/qmail-smtpd
for i in 2 1 0
do
  output=`ps ax | grep qmail-s[em][nt][dp] | sed -e "/^.*grep.*$/d" -e "/^.*tail.*$/d" -e "/^.*supervise.*$/d" -e "/^.*tcpserver.*$/d"`
  if [ -z "${output}" ]
  then
    break
  else
    ${SLEEP_PATH} 1
  fi
done
if [ ! -z "${output}" ]
then
  svc -k /service/qmail-send
  svc -k /service/qmail-smtpd
fi
svc -u /service/qmail-send
svc -u /service/qmail-smtpd

