# This script tests recipient validation using the flowchart in the
# documentation folder.  It follows this path through the flowchart:
#   RELAYCLIENT is not set (f01__is_relayclient_set)
#   recipient has domain name (f02__does_recipient_have_domain_name)
#   domain is in rcpthosts (f05__is_domain_in_rcpthosts)
#   recipient does not contain a(nother) percent sign (f07__does_recipient_contain_percent)
#   domain is not in locals (f09__is_domain_in_locals)
#   domain is in virtualdomains (f10__is_domain_in_virtualdomains)
# The virtualdomains file used by qmail does not contain the domain but qmail-virtualdomains-file tells spamdyke to use a different file
#   cdb contains recipient exact match (f12__does_cdb_contain_exact_match)
#   home directory exists (f14__does_exact_match_home_directory_exist)
#   home directory is readable (f15__is_exact_match_home_directory_readable)
#   home directory is not world-writable (f16__is_exact_match_home_directory_world_writable)
#   home directory is not sticky (f17__is_exact_match_home_directory_sticky)
#   constructed filename does not exist (f18__does_exact_match_filename_exist)
#   constructed filename exists (f19__does_exact_match_filename_exist_default)
#   file is not world-writeable (f33__is_file_world_writable)
#   file is not empty (f34__is_file_empty)
#   file is not owner-executable (f35__is_file_owner_executable)
#   first line is not blank (f36__is_first_line_blank)
#   more lines in file (f37__more_lines_in_file)
#   line in file is not a comment (f38__is_line_comment)
#   line in file is not an mbox (f39__is_line_mbox)
#   line in file is a maildir (f41__is_line_maildir)
#   maildir exists (f42__does_maildir_exist)
# The message should be accepted

export TCPREMOTEIP=11.22.33.44
unset GREP_TARGET
unset REJECT_GREP_TARGET
unset TEMP_FILENAME
unset HOMEDIR
unset DASH
unset EXT
unset REAL_RELAYCLIENT
unset REAL_USERNAME
unset REAL_DOMAIN
unset REAL_ADDRESS
unset TO_USERNAME
unset TO_DOMAIN
unset TO_ADDRESS
unset TEMP_USERNAME
unset WILDCARD_DOT
unset WILDCARD_DASH
unset WILDCARD_DASH2
unset STRLEN_REAL_USERNAME
unset PATTERN
unset WILDCARD_ADDITION
unset IN_MORERCPTHOSTS
unset IN_RCPTHOSTS
unset PERCENTHACK_USERNAME_ADDITION
unset VIRTUALDOMAINS_USERNAME_ADDITION
FROM_ADDRESS=test-${TEST_NUM}.cqcpc.ocmcl@example.com
REAL_USERNAME=s-ckc
REAL_DOMAIN=jc.test
REAL_ADDRESS=${REAL_USERNAME}@${REAL_DOMAIN}
TO_USERNAME=${REAL_USERNAME}
TO_DOMAIN=${REAL_DOMAIN}
TO_ADDRESS=${REAL_ADDRESS}
TEMP_USERNAME=${REAL_USERNAME}
VALID_DOMAIN=domain-ichcg-cfcdc-ccbca.test
VALID_USERNAME=user-cacbc-ccdcf-cgchc
VALID_ADDRESS=${VALID_USERNAME}@${VALID_DOMAIN}
VALID_HOMEDIR=${TMPDIR}/${TEST_NUM}-valid_home
VALID_GREP_TARGET=${VALID_HOMEDIR}/mbox
mkdir -p ${VALID_HOMEDIR}
chmod 700 ${VALID_HOMEDIR}
chown -R $6 ${VALID_HOMEDIR}
chgrp -R $7 ${VALID_HOMEDIR}
echo ${VALID_HOMEDIR}/mbox > ${VALID_HOMEDIR}/.qmail
touch ${VALID_HOMEDIR}/mbox
chmod 600 ${VALID_HOMEDIR}/mbox
chown -R $6 ${VALID_HOMEDIR}/mbox
chgrp -R $7 ${VALID_HOMEDIR}/mbox


################################################################################
# RELAYCLIENT is not set (f01__is_relayclient_set)
LEVEL_f01__is_relayclient_set=1
export -n RELAYCLIENT

################################################################################
# domain is in rcpthosts (f05__is_domain_in_rcpthosts)
LEVEL_f05__is_domain_in_rcpthosts=1
IN_RCPTHOSTS="true"
cp /var/qmail/control/rcpthosts ${TMPDIR}/${TEST_NUM}-f05_rcpthosts.bu.${LEVEL_f05__is_domain_in_rcpthosts}
echo ${REAL_DOMAIN} > ${TMPDIR}/${TEST_NUM}-f05_rcpthosts.${LEVEL_f05__is_domain_in_rcpthosts}
cp ${TMPDIR}/${TEST_NUM}-f05_rcpthosts.${LEVEL_f05__is_domain_in_rcpthosts} /var/qmail/control/rcpthosts

################################################################################
# domain is not in locals (f09__is_domain_in_locals)
LEVEL_f09__is_domain_in_locals=1
cp /var/qmail/control/locals ${TMPDIR}/${TEST_NUM}-f09_locals.bu.${LEVEL_f09__is_domain_in_locals}
cat ${TMPDIR}/${TEST_NUM}-f09_locals.bu.${LEVEL_f09__is_domain_in_locals} | sed -e "/^${REAL_DOMAIN}$/d" > ${TMPDIR}/${TEST_NUM}-f09_locals.${LEVEL_f09__is_domain_in_locals}
echo ${VALID_DOMAIN} >> ${TMPDIR}/${TEST_NUM}-f09_locals.${LEVEL_f09__is_domain_in_locals}
cp ${TMPDIR}/${TEST_NUM}-f09_locals.${LEVEL_f09__is_domain_in_locals} /var/qmail/control/locals

################################################################################
# domain is in virtualdomains (f10__is_domain_in_virtualdomains)
LEVEL_f10__is_domain_in_virtualdomains=1
VIRTUALDOMAINS_REAL_USERNAME[${LEVEL_f10__is_domain_in_virtualdomains}]=${REAL_USERNAME}
cp /var/qmail/control/virtualdomains ${TMPDIR}/${TEST_NUM}-f10_virtualdomains.bu.${LEVEL_f10__is_domain_in_virtualdomains}
echo "${REAL_DOMAIN}:${REAL_DOMAIN}-" > ${TMPDIR}/${TEST_NUM}-f10_virtualdomains.${LEVEL_f10__is_domain_in_virtualdomains}
cp ${TMPDIR}/${TEST_NUM}-f10_virtualdomains.${LEVEL_f10__is_domain_in_virtualdomains} /var/qmail/control/virtualdomains
VIRTUALDOMAINS_USERNAME_ADDITION=${REAL_DOMAIN}--
REAL_USERNAME=${VIRTUALDOMAINS_USERNAME_ADDITION}${REAL_USERNAME}
REAL_ADDRESS=${REAL_USERNAME}@${REAL_DOMAIN}

################################################################################
# cdb contains recipient exact match (f12__does_cdb_contain_exact_match)
LEVEL_f12__does_cdb_contain_exact_match=1
cp /var/qmail/users/assign ${TMPDIR}/${TEST_NUM}-f12_assign.bu.${LEVEL_f12__does_cdb_contain_exact_match}
cat ${TMPDIR}/${TEST_NUM}-f12_assign.bu.${LEVEL_f12__does_cdb_contain_exact_match} | sed -e "/^=${REAL_USERNAME}:.*$/d" -e "/^\.$/d" > ${TMPDIR}/${TEST_NUM}-f12_assign.${LEVEL_f12__does_cdb_contain_exact_match}
PATTERN=${REAL_USERNAME}
USER=${REAL_USERNAME}
HOMEDIR=${TMPDIR}/${TEST_NUM}-home.${LEVEL_f12__does_cdb_contain_exact_match}.8v8u8
DASH=-DASH${LEVEL_f12__does_cdb_contain_exact_match}.t8s8r-
DASH_EMPTY=""
FILE_EXT=EXT${LEVEL_f12__does_cdb_contain_exact_match}.8q8p8
EXT=`echo ${FILE_EXT} | tr '[:upper:]' '[:lower:]' | sed -e "s/\./:/g"`
TEMP_FILENAME=${HOMEDIR}/.qmail${DASH}${EXT}
echo "=${PATTERN}:${REAL_USERNAME}:$6:$7:${HOMEDIR}:${DASH}:${FILE_EXT}:" >> ${TMPDIR}/${TEST_NUM}-f12_assign.${LEVEL_f12__does_cdb_contain_exact_match}
echo "=${VALID_USERNAME}:${VALID_USERNAME}:$6:$7:${VALID_HOMEDIR}:::" >> ${TMPDIR}/${TEST_NUM}-f12_assign.${LEVEL_f12__does_cdb_contain_exact_match}
echo "." >> ${TMPDIR}/${TEST_NUM}-f12_assign.${LEVEL_f12__does_cdb_contain_exact_match}
cp ${TMPDIR}/${TEST_NUM}-f12_assign.${LEVEL_f12__does_cdb_contain_exact_match} /var/qmail/users/assign
/var/qmail/bin/qmail-newu

################################################################################
# home directory exists (f14__does_exact_match_home_directory_exist)
LEVEL_f14__does_exact_match_home_directory_exist=1
mkdir -p ${HOMEDIR}
chmod 700 ${HOMEDIR}
chown -R $6 ${HOMEDIR}
chgrp -R $7 ${HOMEDIR}

################################################################################
# home directory is readable (f15__is_exact_match_home_directory_readable)
LEVEL_f15__is_exact_match_home_directory_readable=1
chmod u+r ${HOMEDIR}

################################################################################
# home directory is not world-writable (f16__is_exact_match_home_directory_world_writable)
LEVEL_f16__is_exact_match_home_directory_world_writable=1
eval `${STMODE_PATH} ${HOMEDIR}`
EXACT_HOMEDIR_WRITABLE_MODE[${LEVEL_f16__is_exact_match_home_directory_world_writable}]=${st_mode}
chmod o-w ${HOMEDIR}

################################################################################
# home directory is not sticky (f17__is_exact_match_home_directory_sticky)
LEVEL_f17__is_exact_match_home_directory_sticky=1
eval `${STMODE_PATH} ${HOMEDIR}`
EXACT_HOMEDIR_STICKY_MODE[${LEVEL_f17__is_exact_match_home_directory_sticky}]=${st_mode}
chmod -t ${HOMEDIR}

################################################################################
# constructed filename does not exist (f18__does_exact_match_filename_exist)
LEVEL_f18__does_exact_match_filename_exist=1
TEMP_FILENAME=${HOMEDIR}/.qmail${DASH}default

################################################################################
# constructed filename exists (f19__does_exact_match_filename_exist_default)
LEVEL_f19__does_exact_match_filename_exist_default=1
touch ${TEMP_FILENAME}
chown $6 ${TEMP_FILENAME}
chgrp $7 ${TEMP_FILENAME}

################################################################################
# file is not world-writeable (f33__is_file_world_writable)
LEVEL_f33__is_file_world_writable=1
eval `${STMODE_PATH} ${TEMP_FILENAME}`
FILE_WRITABLE_MODE[${LEVEL_f33__is_file_world_writable}]=${st_mode}
chmod o-w ${TEMP_FILENAME}

################################################################################
# file is not owner-executable (f35__is_file_owner_executable)
LEVEL_f35__is_file_owner_executable=1
eval `${STMODE_PATH} ${TEMP_FILENAME}`
FILE_EXECUTABLE_MODE[${LEVEL_f35__is_file_owner_executable}]=${st_mode}
chmod u-x ${TEMP_FILENAME}

################################################################################
# line in file is a maildir (f41__is_line_maildir)
LEVEL_f41__is_line_maildir=1
echo "./maildir.${LEVEL_f41__is_line_maildir}/" >> ${TEMP_FILENAME}

################################################################################
# maildir exists (f42__does_maildir_exist)
LEVEL_f42__does_maildir_exist=1
GREP_TARGET=${HOMEDIR}/maildir.${LEVEL_f41__is_line_maildir}/new
mkdir -p ${HOMEDIR}/maildir.${LEVEL_f41__is_line_maildir}
mkdir ${HOMEDIR}/maildir.${LEVEL_f41__is_line_maildir}/cur
mkdir ${HOMEDIR}/maildir.${LEVEL_f41__is_line_maildir}/new
mkdir ${HOMEDIR}/maildir.${LEVEL_f41__is_line_maildir}/tmp
chmod -R 700 ${HOMEDIR}/maildir.${LEVEL_f41__is_line_maildir}
chown -R $6 ${HOMEDIR}/maildir.${LEVEL_f41__is_line_maildir}
chgrp -R $7 ${HOMEDIR}/maildir.${LEVEL_f41__is_line_maildir}

################################################################################
# final steps
if [ -z "${GREP_TARGET}" ]
then
  GREP_TARGET=${REJECT_GREP_TARGET}
fi

################################################################################
# empty queue
echo Emptying queue and restarting qmail...
svc -d /service/qmail-send
svc -d /service/qmail-smtpd
for i in 2 1 0
do
  output=`ps ax | grep qmail-s[em][nt][dp] | sed -e "/^.*grep.*$/d" -e "/^.*tail.*$/d" -e "/^.*supervise.*$/d" -e "/^.*tcpserver.*$/d"`
  if [ -z "${output}" ]
  then
    break
  else
    ${SLEEP_PATH} 1
  fi
done
if [ ! -z "${output}" ]
then
  svc -k /service/qmail-send
  svc -k /service/qmail-smtpd
fi
rm -f `find /var/qmail/queue/mess /var/qmail/queue/info /var/qmail/queue/local /var/qmail/queue/remote /var/qmail/queue/bounce /var/qmail/queue/intd /var/qmail/queue/todo -type f -print`
svc -u /service/qmail-send
svc -u /service/qmail-smtpd


################################################################################
# create input file
echo "HELO me" > ${TMPDIR}/${TEST_NUM}-input.txt
echo "MAIL FROM:<${FROM_ADDRESS}>" >> ${TMPDIR}/${TEST_NUM}-input.txt
echo "RCPT TO:<${TO_ADDRESS}>" >> ${TMPDIR}/${TEST_NUM}-input.txt
echo "DATA" >> ${TMPDIR}/${TEST_NUM}-input.txt
echo "To: <${REAL_ADDRESS}>\r\nFrom: <${FROM_ADDRESS}>\r\nSubject: ${TEST_NUM}\r\n\r\nFOO!\r\n.\r\n" >> ${TMPDIR}/${TEST_NUM}-input.txt
echo "QUIT" >> ${TMPDIR}/${TEST_NUM}-input.txt

cp /var/qmail/control/virtualdomains ${TMPDIR}/${TEST_NUM}-virtualdomains
rm /var/qmail/control/virtualdomains
touch /var/qmail/control/virtualdomains

################################################################################
# run test
echo "${SENDRECV_PATH} -t 30 -r 221 -w 0 -- ${SPAMDYKE_PATH} --log-target stderr -linfo --recipient-validation-command \"${SPAMDYKE_QRV_PATH} --qmail-virtualdomains-file ${TMPDIR}/${TEST_NUM}-virtualdomains\" --reject-recipient invalid --reject-recipient unavailable ${SMTPDUMMY_PATH} < ${TMPDIR}/${TEST_NUM}-input.txt > ${TMPDIR}/${TEST_NUM}-output.txt 2>&1"
${SENDRECV_PATH} -t 30 -r 221 -w 0 -- ${SPAMDYKE_PATH} --log-target stderr -linfo --reject-recipient invalid --recipient-validation-command "${SPAMDYKE_QRV_PATH} --qmail-virtualdomains-file ${TMPDIR}/${TEST_NUM}-virtualdomains" --reject-recipient unavailable ${SMTPDUMMY_PATH} < ${TMPDIR}/${TEST_NUM}-input.txt > ${TMPDIR}/${TEST_NUM}-output.txt 2>&1

output=`grep "ALLOWED from: ${FROM_ADDRESS} to: ${TO_ADDRESS}" ${TMPDIR}/${TEST_NUM}-output.txt`
if [ ! -z "${output}" ]
then
  unset outcome
  output=`grep "250 DATA END received" ${TMPDIR}/${TEST_NUM}-output.txt`
  if [ ! -z "${output}" ]
  then
    outcome="success"
  else
    echo OUTPUT IN tmp/${TEST_NUM}-output.txt:
    cat ${TMPDIR}/${TEST_NUM}-output.txt

    outcome="failure"
  fi
else
  echo OUTPUT IN tmp/${TEST_NUM}-output.txt:
  cat ${TMPDIR}/${TEST_NUM}-output.txt

  outcome="failure"
fi

################################################################################
# file is not owner-executable (f35__is_file_owner_executable)
chmod ${FILE_EXECUTABLE_MODE[${LEVEL_f35__is_file_owner_executable}]} ${TEMP_FILENAME}
LEVEL_f35__is_file_owner_executable=0

################################################################################
# file is not world-writeable (f33__is_file_world_writable)
chmod ${FILE_WRITABLE_MODE[${LEVEL_f33__is_file_world_writable}]} ${TEMP_FILENAME}
LEVEL_f33__is_file_world_writable=0

################################################################################
# home directory is not sticky (f17__is_exact_match_home_directory_sticky)
chmod ${EXACT_HOMEDIR_STICKY_MODE[${LEVEL_f17__is_exact_match_home_directory_sticky}]} ${HOMEDIR}
LEVEL_f17__is_exact_match_home_directory_sticky=0

################################################################################
# home directory is not world-writable (f16__is_exact_match_home_directory_world_writable)
chmod ${EXACT_HOMEDIR_WRITABLE_MODE[${LEVEL_f16__is_exact_match_home_directory_world_writable}]} ${HOMEDIR}
LEVEL_f16__is_exact_match_home_directory_world_writable=0

################################################################################
# cdb contains recipient exact match (f12__does_cdb_contain_exact_match)
cp ${TMPDIR}/${TEST_NUM}-f12_assign.bu.${LEVEL_f12__does_cdb_contain_exact_match} /var/qmail/users/assign
/var/qmail/bin/qmail-newu
LEVEL_f12__does_cdb_contain_exact_match=0

################################################################################
# domain is in virtualdomains (f10__is_domain_in_virtualdomains)
REAL_USERNAME=${VIRTUALDOMAINS_REAL_USERNAME[${LEVEL_f10__is_domain_in_virtualdomains}]}
REAL_ADDRESS=${REAL_USERNAME}@${REAL_DOMAIN}
cp ${TMPDIR}/${TEST_NUM}-f10_virtualdomains.bu.${LEVEL_f10__is_domain_in_virtualdomains} /var/qmail/control/virtualdomains
LEVEL_f10__is_domain_in_virtualdomains=0

################################################################################
# domain is not in locals (f09__is_domain_in_locals)
cp ${TMPDIR}/${TEST_NUM}-f09_locals.bu.${LEVEL_f09__is_domain_in_locals} /var/qmail/control/locals
LEVEL_f09__is_domain_in_locals=0

################################################################################
# domain is in rcpthosts (f05__is_domain_in_rcpthosts)
cp ${TMPDIR}/${TEST_NUM}-f05_rcpthosts.bu.${LEVEL_f05__is_domain_in_rcpthosts} /var/qmail/control/rcpthosts
LEVEL_f05__is_domain_in_rcpthosts=0


echo Restarting qmail...
svc -d /service/qmail-send
svc -d /service/qmail-smtpd
for i in 2 1 0
do
  output=`ps ax | grep qmail-s[em][nt][dp] | sed -e "/^.*grep.*$/d" -e "/^.*tail.*$/d" -e "/^.*supervise.*$/d" -e "/^.*tcpserver.*$/d"`
  if [ -z "${output}" ]
  then
    break
  else
    ${SLEEP_PATH} 1
  fi
done
if [ ! -z "${output}" ]
then
  svc -k /service/qmail-send
  svc -k /service/qmail-smtpd
fi
svc -u /service/qmail-send
svc -u /service/qmail-smtpd

